#include <iostream>
#include <time.h>
#include <unistd.h>
#include <pthread.h>
#include <QDateTime>
#include <QDebug>
#include <QMutex>
#include <QPainter>
#include <QImage>
#include <QPixmap>
#include <QTextCodec>
#include <QDir>
#include "qhyccddemo.h"
#include "ui_qhyccddemo.h"
#include "qhyccd.h"
#include "math.h"
#include "fitsio.h"

using namespace std;

qhyccd_handle *camhandle = NULL;
qhyccd_handle *camhandle_2 = NULL;
struct CAM CamInfo;
struct CAM CamInfo_2;

QMutex ShowImgMutex;
QMutex ShowImgMutex_2;

QHYCCDDemo::QHYCCDDemo(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::QHYCCDDemo)
{
    ui->setupUi(this);

    CamInfo.Frame = 0;
    CamInfo_2.Frame = 0;
    camhandle = NULL;
    camhandle_2 = NULL;
    CamInfo.ImgData = NULL;
    CamInfo_2.ImgData = NULL;

    ret = InitQHYCCDResource();
    if(ret == QHYCCD_ERROR)
    {
        ui->textBrowser_Info->append("InitQHYCCDResource failed ...");
        ui->textBrowser_Info_2->append("InitQHYCCDResource failed ...");
        return;
    }
}

QHYCCDDemo::~QHYCCDDemo()
{
    ret = ReleaseQHYCCDResource();
    if(ret == QHYCCD_ERROR)
    {
        ui->textBrowser_Info->append("ReleaseQHYCCDResource() failed ...");
        ui->textBrowser_Info_2->append("ReleaseQHYCCDResource() failed ...");
        return;
    }

    delete ui;
}

void QHYCCDDemo::closeEvent(QCloseEvent *event)
{
    if(camhandle != NULL)
    {
        ret = CloseQHYCCD(camhandle);
        if(ret == QHYCCD_ERROR)
        {
            ui->textBrowser_Info->append("CloseQHYCCD() failed ...");
            return;
        }

        camhandle = NULL;
        if(CamInfo.ImgData)
        {
            free(CamInfo.ImgData);
            CamInfo.ImgData = NULL;
        }
        if(CamInfo.ShowImgData)
        {
            free(CamInfo.ShowImgData);
            CamInfo.ShowImgData = NULL;
        }
    }

    if(camhandle_2 != NULL)
    {
        ret = CloseQHYCCD(camhandle_2);
        if(ret == QHYCCD_ERROR)
        {
            ui->textBrowser_Info_2->append("CloseQHYCCD() failed ...");
            return;
        }

        camhandle_2 = NULL;
        if(CamInfo_2.ImgData)
        {
            free(CamInfo_2.ImgData);
            CamInfo_2.ImgData = NULL;
        }
        if(CamInfo_2.ShowImgData)
        {
            free(CamInfo_2.ShowImgData);
            CamInfo_2.ShowImgData = NULL;
        }
    }

    this->setWindowTitle("QHYCCD Qt Demo");
}

void QHYCCDDemo::on_pBtn_ScanCamera_clicked()
{
    uint32_t num = 0;
    num = ScanQHYCCD();
    if(num == 0)
    {
        ui->textBrowser_Info->append("ScanQHYCCD no Camera be found ...");
        return;
    }

    ui->textBrowser_Info->append(QString("Found %1 Camera ...").arg(num));
    QApplication::processEvents();

    QStringList idlist;
    char id[40] = {0};
    for(uint32_t i = 0; i < num; i ++)
    {
        memset(id, 0, 40);
        ret = GetQHYCCDId(i, id);
        if(ret == QHYCCD_SUCCESS)
            idlist.append(id);
        else
            ui->textBrowser_Info->append(QString("GetQHYCCDId() %1 failed ...").arg(i));
    }

    ui->comboBox_CameraList->blockSignals(true);
    ui->comboBox_CameraList->clear();
    ui->comboBox_CameraList->blockSignals(false);
    ui->comboBox_CameraList->addItems(idlist);
}

void QHYCCDDemo::on_comboBox_CameraList_currentTextChanged(const QString &arg1)
{
    ret = GetQHYCCDModel(arg1.toLatin1().data(), CamInfo.Model);
    if(ret == QHYCCD_ERROR)
    {
        ui->textBrowser_Info->append("GetQHYCCDModel failed ...");
        return;
    }

    camhandle = OpenQHYCCD(arg1.toLatin1().data());
    if(camhandle == NULL)
    {
        ui->textBrowser_Info->append("OpenQHYCCD failed ...");
        return;
    }

    uint32_t readModeNumber = 0;
    ret = GetQHYCCDNumberOfReadModes(camhandle, &readModeNumber);
    QStringList readModeList;
    for(uint32_t i = 0;i < readModeNumber; i++)
    {
        char readModeName[20] = {0};
        ret = GetQHYCCDReadModeName(camhandle, i, readModeName);
        if(ret == QHYCCD_SUCCESS)
        {
            readModeList.append(readModeName);
        }
    }

    ui->comboBox_ReadModeList->blockSignals(true);
    ui->comboBox_ReadModeList->clear();
    ui->comboBox_ReadModeList->addItems(readModeList);
    ui->comboBox_ReadModeList->blockSignals(false);
}

void QHYCCDDemo::on_pBtn_Connect_clicked()
{
    ui->textBrowser_Info->append("Start to connect Camera ...");
    QApplication::processEvents();

    camhandle = OpenQHYCCD(ui->comboBox_CameraList->currentText().toLatin1().data());
    if(camhandle == NULL)
    {
        ui->textBrowser_Info->append("OpenQHYCCD failed ...");
        return;
    }

    uint32_t readmode = static_cast<uint32_t>(ui->comboBox_ReadModeList->currentIndex());
    ret = SetQHYCCDReadMode(camhandle, readmode); //Setup Read Mode
    if(ret == QHYCCD_ERROR)
    {
        ui->textBrowser_Info->append("SetQHYCCDReadMode failed ...");
        return;
    }

    ret = SetQHYCCDStreamMode(camhandle, 0); //Setup Single Mode
    if(ret == QHYCCD_ERROR)
    {
        ui->textBrowser_Info->append("SetQHYCCDStreamMode failed ...");
        return;
    }

    ret = InitQHYCCD(camhandle);
    if(ret == QHYCCD_ERROR)
    {
        ui->textBrowser_Info->append("InitQHYCCD Failed ...");
        return;
    }

    ui->comboBox_Bin->blockSignals(true);
    ui->comboBox_Bin->clear();
    ui->comboBox_Bin->addItem("1x1");
    if(IsQHYCCDControlAvailable(camhandle, CAM_BIN2X2MODE) == QHYCCD_SUCCESS) ui->comboBox_Bin->addItem("2x2");
    if(IsQHYCCDControlAvailable(camhandle, CAM_BIN3X3MODE) == QHYCCD_SUCCESS) ui->comboBox_Bin->addItem("3x3");
    if(IsQHYCCDControlAvailable(camhandle, CAM_BIN4X4MODE) == QHYCCD_SUCCESS) ui->comboBox_Bin->addItem("4x4");
    ui->comboBox_Bin->setCurrentText("1x1");
    ui->comboBox_Bin->blockSignals(false);

    ret = GetQHYCCDChipInfo(camhandle, &CamInfo.ChipW, &CamInfo.ChipH, &CamInfo.ImageW, &CamInfo.ImageH, &CamInfo.PixelW, &CamInfo.PixelH, &CamInfo.ImageB);
    if(ret == QHYCCD_ERROR)
    {
        ui->textBrowser_Info->append("GetQHYCCDChipInfo failed ...");
        return;
    }

    CamInfo.binx = 1;
    CamInfo.biny = 1;
    ret = SetQHYCCDBinMode(camhandle, 1, 1);
    if(ret == QHYCCD_ERROR)
    {
        ui->textBrowser_Info->append("SetQHYCCDBinMode failed ...");
        return;
    }

    ret = SetQHYCCDResolution(camhandle, 0, 0, CamInfo.ImageW, CamInfo.ImageH);
    if(ret == QHYCCD_ERROR)
    {
        ui->textBrowser_Info->append("SetQHYCCDResolution failed ...");
        return;
    }

    ui->comboBox_Format->blockSignals(true);
    ui->comboBox_Format->clear();
    if(IsQHYCCDControlAvailable(camhandle, CAM_16BITS)   == QHYCCD_SUCCESS &&
       IsQHYCCDControlAvailable(camhandle, CAM_IS_COLOR) == QHYCCD_SUCCESS) ui->comboBox_Format->addItem("RAW16");
    if(IsQHYCCDControlAvailable(camhandle, CAM_8BITS)    == QHYCCD_SUCCESS &&
       IsQHYCCDControlAvailable(camhandle, CAM_IS_COLOR) == QHYCCD_ERROR)   ui->comboBox_Format->addItem("MONO16");
    if(IsQHYCCDControlAvailable(camhandle, CAM_IS_COLOR) == QHYCCD_SUCCESS)
        ui->comboBox_Format->setCurrentText("RAW16");
    else
        ui->comboBox_Format->setCurrentText("MONO16");
    ui->comboBox_Format->blockSignals(false);

    ret = SetQHYCCDDebayerOnOff(camhandle, false);
    if(ret == QHYCCD_ERROR)
    {
        ui->textBrowser_Info->append("SetQHYCCDDebayerOnOff failed ...");
        return;
    }

    ret = SetQHYCCDParam(camhandle, CONTROL_TRANSFERBIT, 16);
    if(ret == QHYCCD_ERROR)
    {
        ui->textBrowser_Info->append("SetQHYCCDParam(CONTROL_TRANSFERBIT) failed ...");
        return;
    }

    ui->spinBox_Exposure->blockSignals(true);
    ui->comboBox_Unit->blockSignals(true);
    ui->hSlider_Exposure->blockSignals(true);
    ui->spinBox_Exposure->setValue(20);
    ui->hSlider_Exposure->setValue(20);
    ui->comboBox_Unit->clear();
    ui->comboBox_Unit->addItem("us");
    ui->comboBox_Unit->addItem("ms");
    ui->comboBox_Unit->addItem("s");
    ui->comboBox_Unit->setCurrentText("ms");
    ui->spinBox_Exposure->blockSignals(false);
    ui->comboBox_Unit->blockSignals(false);
    ui->hSlider_Exposure->blockSignals(false);

    ret = SetQHYCCDParam(camhandle, CONTROL_EXPOSURE, 20000.0);
    if(ret == QHYCCD_ERROR)
    {
        ui->textBrowser_Info->append("SetQHYCCDParam(CONTROL_EXPOSURE) failed ...");
        return;
    }

    double min, max, step;

    ui->hSlider_Gain->blockSignals(true);
    ui->spinBox_Gain->blockSignals(true);
    ret = GetQHYCCDParamMinMaxStep(camhandle, CONTROL_GAIN, &min, &max, &step);
    ui->hSlider_Gain->setMinimum(static_cast<int>(min));
    ui->hSlider_Gain->setMaximum(static_cast<int>(max));
    ui->spinBox_Gain->setMinimum(static_cast<int>(min));
    ui->spinBox_Gain->setMaximum(static_cast<int>(max));
    ui->hSlider_Gain->setValue(20);
    ui->spinBox_Gain->setValue(20);
    ui->hSlider_Gain->blockSignals(false);
    ui->spinBox_Gain->blockSignals(false);

    ret = SetQHYCCDParam(camhandle, CONTROL_GAIN, 20);
    if(ret == QHYCCD_ERROR)
    {
        ui->textBrowser_Info->append("SetQHYCCDParam(CONTROL_GAIN) failed ...");
        return;
    }

    ui->hSlider_Offset->blockSignals(true);
    ui->spinBox_Offset->blockSignals(true);
    ret = GetQHYCCDParamMinMaxStep(camhandle, CONTROL_GAIN, &min, &max, &step);
    ui->hSlider_Offset->setMinimum(static_cast<int>(min));
    ui->hSlider_Offset->setMaximum(static_cast<int>(max));
    ui->spinBox_Offset->setMinimum(static_cast<int>(min));
    ui->spinBox_Offset->setMaximum(static_cast<int>(max));
    ui->hSlider_Offset->setValue(20);
    ui->spinBox_Offset->setValue(20);
    ui->hSlider_Offset->blockSignals(false);
    ui->spinBox_Offset->blockSignals(false);

    ret = SetQHYCCDParam(camhandle, CONTROL_OFFSET, 20);
    if(ret == QHYCCD_ERROR)
    {
        ui->textBrowser_Info->append("SetQHYCCDParam(CONTROL_OFFSET) failed ...");
        return;
    }

    CamInfo.ImgData = (unsigned char *)malloc(GetQHYCCDMemLength(camhandle));
    CamInfo.ShowImgData = (unsigned char *)malloc(GetQHYCCDMemLength(camhandle));

    this->setWindowTitle(this->windowTitle() + " | " + ui->comboBox_CameraList->currentText());
    ui->textBrowser_Info->append("Connected Camera,the Camera ID is " + ui->comboBox_CameraList->currentText());
}

void QHYCCDDemo::on_pBtn_Disconnect_clicked()
{
    if(camhandle == NULL) return;

    ret = CloseQHYCCD(camhandle);
    if(ret == QHYCCD_ERROR)
    {
        ui->textBrowser_Info->append("CloseQHYCCD() failed ...");
        return;
    }

    camhandle = NULL;
    if(CamInfo.ImgData)
    {
        free(CamInfo.ImgData);
        CamInfo.ImgData = NULL;
    }
    if(CamInfo.ShowImgData)
    {
        free(CamInfo.ShowImgData);
        CamInfo.ShowImgData = NULL;
    }

    this->setWindowTitle(this->windowTitle().remove(" | " + ui->comboBox_CameraList->currentText()));
    ui->textBrowser_Info->append("Disconnected Camera ...");
}

void QHYCCDDemo::FITwrite(QString filename, int ImageWidth, int ImageHeight, int ImageBpp, int ImageChannel, unsigned char *Buf)
{
    fitsfile *fptr;
    int  status = 0;//, ii, jj;

    int  bitpix = 0, datatype = 0;
    long naxis = 0, nelements = 0;//维数, 数据长度
    long naxesColor[3] = {ImageWidth, ImageHeight, ImageChannel};
    long fpixel = 1;

    if(ImageBpp == 8 && ImageChannel == 1)
    {
        bitpix    = BYTE_IMG;
        naxis     = 2;
        nelements = naxesColor[0] * naxesColor[1];
        datatype  = TBYTE;
    }
    else if(ImageBpp == 16 && ImageChannel == 1)
    {
        bitpix    = USHORT_IMG;
        naxis     = 2;
        nelements = naxesColor[0] * naxesColor[1];
        datatype  = TUSHORT;
    }
    else if(ImageBpp == 8 && ImageChannel == 3)
    {
        bitpix    = BYTE_IMG;
        naxis     = 3;
        nelements = naxesColor[0] * naxesColor[1] * naxesColor[2];
        datatype  = TBYTE;
    }

    //string holds the text converted to Unicode, 支持中文路径
    QTextCodec *codec = QTextCodec::codecForName("GB18030");
    QByteArray encodedString = codec->fromUnicode(filename);

    fits_create_file(&fptr, encodedString.constData(), &status);

    fits_create_img(fptr, bitpix, naxis, naxesColor, &status);

    unsigned char *bufConvert = (unsigned char *)malloc(ImageWidth*ImageHeight*ImageBpp*ImageChannel/8);
    if(ImageChannel == 3)
        for(int i = 0; i < ImageChannel; i ++)
            for(int j = 0; j < ImageWidth*ImageHeight; j++)
                bufConvert[j+ImageWidth*ImageHeight*i] = Buf[3*j+ImageChannel-i-1];
    else
        memcpy(bufConvert, Buf, ImageWidth*ImageHeight*ImageBpp*ImageChannel/8);

    fits_write_img(fptr, datatype, fpixel, nelements, &bufConvert[0], &status);

    fits_close_file(fptr, &status);

    free(bufConvert);
}

void QHYCCDDemo::processImageData()
{
    ShowImgMutex.lock();

    unsigned char *ImageTemp = (unsigned char *)malloc(CamInfo.ShowImgW*CamInfo.ShowImgH*CamInfo.ImgC);

    if(CamInfo.ShowImgB == 8 && CamInfo.ShowImgC == 1)
    {
        for(uint32_t i = 0; i < CamInfo.ShowImgW*CamInfo.ShowImgH; i++)
        {
            ImageTemp[i] = CamInfo.ShowImgData[i];
        }
    }
    else if(CamInfo.ShowImgB == 16 && CamInfo.ShowImgC == 1)
    {
        for(uint32_t i = 0; i < CamInfo.ShowImgW*CamInfo.ShowImgH; i++)
        {
            ImageTemp[i] = CamInfo.ShowImgData[2*i+1];
        }
    }
    else if(CamInfo.ShowImgB == 8 && CamInfo.ShowImgC == 3)
    {
        for(uint32_t i = 0; i < CamInfo.ShowImgW*CamInfo.ShowImgH*CamInfo.ShowImgC; i+=3)
        {
            ImageTemp[i]   = CamInfo.ShowImgData[i+2];
            ImageTemp[i+1] = CamInfo.ShowImgData[i+1];
            ImageTemp[i+2] = CamInfo.ShowImgData[i];
        }
    }
    else if(CamInfo.ShowImgB == 16 && CamInfo.ShowImgC == 3)
    {
        for(uint32_t i = 0; i < CamInfo.ShowImgW*CamInfo.ShowImgH*CamInfo.ShowImgC; i+=3)
        {
            ImageTemp[i]   = CamInfo.ShowImgData[2*i+5];
            ImageTemp[i+1] = CamInfo.ShowImgData[2*i+3];
            ImageTemp[i+2] = CamInfo.ShowImgData[2*i+1];
        }
    }

    QImage::Format format = CamInfo.ShowImgC == 1 ? QImage::Format_Grayscale8 : QImage::Format_RGB888;
    QImage *ImageOri = new QImage(ImageTemp, CamInfo.ShowImgW, CamInfo.ShowImgH, CamInfo.ShowImgW*CamInfo.ShowImgC, format);
    QImage ImageShow = ImageOri->scaled(ui->label_Image->width(), ui->label_Image->height(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
    ui->label_Image->setPixmap(QPixmap::fromImage(ImageShow));

    delete ImageOri;
    free(ImageTemp);

    ShowImgMutex.unlock();

    ui->label_Frame->setText("Frame : " + QString::number(CamInfo.Frame));
    ui->textBrowser_Info->append("Capture image success ...");

    if(ui->checkBox_AutoSave->isChecked())
    {
        QString savePath = QCoreApplication::applicationDirPath();
        QString fileName = savePath + "/" + CamInfo.Model + "-" +
                QDateTime::currentDateTime().toString("yyyyMMddhhmmss") + ".fit";
        FITwrite(fileName, CamInfo.ShowImgW, CamInfo.ShowImgH, CamInfo.ShowImgB, CamInfo.ShowImgC, CamInfo.ShowImgData);
        ui->textBrowser_Info->append("Save as to FIT ...");
    }
}

void QHYCCDDemo::on_pBtn_Capture_clicked()
{
    if(camhandle == NULL) return;

    //for some cameras, must setup bin and resolution in every capture
    this->on_comboBox_Bin_currentTextChanged(ui->comboBox_Bin->currentText());

    ui->textBrowser_Info->append("Start capturing ...");
    QApplication::processEvents();

    ret = ExpQHYCCDSingleFrame(camhandle);
    if(ret == QHYCCD_ERROR)
    {
        ui->textBrowser_Info->append("ExpQHYCCDSingleFrame() failed ...");
        return;
    }

    singleThread = new SingleModeThread(this);
    connect(singleThread, SIGNAL(finished()), singleThread, SLOT(deleteLater()));
    connect(singleThread, SIGNAL(gotImageData()), this, SLOT(processImageData()));
    singleThread->start();
}


void QHYCCDDemo::on_pBtn_Cancel_clicked()
{
    if(camhandle == NULL) return;

    ret = CancelQHYCCDExposingAndReadout(camhandle);
    if(ret == QHYCCD_ERROR)
    {
        ui->textBrowser_Info->append("CancelQHYCCDExposingAndReadout() failed ...");
        return;
    }
}


void QHYCCDDemo::on_comboBox_Format_currentTextChanged(const QString &arg1)
{
    if(arg1 == "RAW8" || arg1 == "MONO8")
    {
        ret = SetQHYCCDDebayerOnOff(camhandle, false);
        ret = SetQHYCCDParam(camhandle, CONTROL_TRANSFERBIT, 8.0);
    }
    else if(arg1 == "RAW16" || arg1 == "MONO16")
    {
        ret = SetQHYCCDDebayerOnOff(camhandle, false);
        ret = SetQHYCCDParam(camhandle, CONTROL_TRANSFERBIT, 16.0);
    }
    else if(arg1 == "RGB8")
    {
        ret = SetQHYCCDDebayerOnOff(camhandle, true);
        ret = SetQHYCCDParam(camhandle, CONTROL_TRANSFERBIT, 8.0);
    }

    if(ret == QHYCCD_SUCCESS)
        ui->textBrowser_Info->append("Setup " + arg1 + " success ...");
    else
        ui->textBrowser_Info->append("Setup " + arg1 + " failed ...");
}

void QHYCCDDemo::on_comboBox_Bin_currentTextChanged(const QString &arg1)
{
    char *debug = QString("QHYCCDDemo | %1 | %2 | arg1 = %3").arg(__FILE__).arg(__FUNCTION__).arg(arg1).toLatin1().data();
    OutputQHYCCDDebug(debug);

    if(arg1 == "1x1")
    {
        ret = SetQHYCCDBinMode(camhandle, 1, 1);
        ret = SetQHYCCDResolution(camhandle, 0, 0, CamInfo.ImageW, CamInfo.ImageH);
    }
    if(arg1 == "2x2")
    {
        ret = SetQHYCCDBinMode(camhandle, 2, 2);
        ret = SetQHYCCDResolution(camhandle, 0, 0, CamInfo.ImageW / 2, CamInfo.ImageH / 2);
    }
    if(arg1 == "3x3")
    {
        ret = SetQHYCCDBinMode(camhandle, 3, 3);
        ret = SetQHYCCDResolution(camhandle, 0, 0, CamInfo.ImageW / 3, CamInfo.ImageH / 3);
    }
    if(arg1 == "4x4")
    {
        ret = SetQHYCCDBinMode(camhandle, 4, 4);
        ret = SetQHYCCDResolution(camhandle, 0, 0, CamInfo.ImageW / 4, CamInfo.ImageH / 4);
    }

    if(ret == QHYCCD_SUCCESS)
        ui->textBrowser_Info->append("Setup " + arg1);
    else
        ui->textBrowser_Info->append("Setup " + arg1 + " failed ...");
}

void QHYCCDDemo::on_spinBox_Exposure_valueChanged(int arg1)
{
    double unit = 1.0;
    if(ui->comboBox_Unit->currentText() == "us")      unit = 1.0;
    else if(ui->comboBox_Unit->currentText() == "ms") unit = 1000.0;
    else if(ui->comboBox_Unit->currentText() == "s")  unit = 1000000.0;

    ret = SetQHYCCDParam(camhandle, CONTROL_EXPOSURE, static_cast<double>(arg1)*unit);
    if(ret == QHYCCD_SUCCESS)
        ui->textBrowser_Info->append("Setup exposure " + QString::number(arg1) + ui->comboBox_Unit->currentText() + " success ...");
    else
        ui->textBrowser_Info->append("Setup exposure " + QString::number(arg1) + ui->comboBox_Unit->currentText() + " failed ...");

    ui->hSlider_Exposure->blockSignals(true);
    ui->hSlider_Exposure->setValue(arg1);
    ui->hSlider_Exposure->blockSignals(false);

}

void QHYCCDDemo::on_comboBox_Unit_currentTextChanged(const QString &arg1)
{
    double unit = 1.0;
    if(arg1 == "us")      unit = 1.0;
    else if(arg1 == "ms") unit = 1000.0;
    else if(arg1 == "s")  unit = 1000000.0;
    int exp = ui->spinBox_Exposure->value();

    ret = SetQHYCCDParam(camhandle, CONTROL_EXPOSURE, static_cast<double>(exp)*unit);
    if(ret == QHYCCD_SUCCESS)
        ui->textBrowser_Info->append("Setup exposure " + QString::number(exp) + arg1 + " success ...");
    else
        ui->textBrowser_Info->append("Setup exposure " + QString::number(exp) + arg1 + " failed ...");
}

void QHYCCDDemo::on_hSlider_Exposure_valueChanged(int value)
{
    double unit = 1.0;
    if(ui->comboBox_Unit->currentText() == "us")      unit = 1.0;
    else if(ui->comboBox_Unit->currentText() == "ms") unit = 1000.0;
    else if(ui->comboBox_Unit->currentText() == "s")  unit = 1000000.0;

    ret = SetQHYCCDParam(camhandle, CONTROL_EXPOSURE, static_cast<double>(value)*unit);
    if(ret == QHYCCD_SUCCESS)
        ui->textBrowser_Info->append("Setup exposure " + QString::number(value) + ui->comboBox_Unit->currentText() + " success ...");
    else
        ui->textBrowser_Info->append("Setup exposure " + QString::number(value) + ui->comboBox_Unit->currentText() + " failed ...");

    ui->spinBox_Exposure->blockSignals(true);
    ui->spinBox_Exposure->setValue(value);
    ui->spinBox_Exposure->blockSignals(false);
}

void QHYCCDDemo::on_hSlider_Gain_valueChanged(int value)
{
    ret = SetQHYCCDParam(camhandle, CONTROL_GAIN, static_cast<double>(value));
    if(ret == QHYCCD_SUCCESS)
        ui->textBrowser_Info->append("Setup gain " + QString::number(value) + " success ...");
    else
        ui->textBrowser_Info->append("Setup gain " + QString::number(value) + " failed ...");

    ui->spinBox_Gain->blockSignals(true);
    ui->spinBox_Gain->setValue(value);
    ui->spinBox_Gain->blockSignals(false);
}

void QHYCCDDemo::on_spinBox_Gain_valueChanged(int arg1)
{
    ret = SetQHYCCDParam(camhandle, CONTROL_GAIN, static_cast<double>(arg1));
    if(ret == QHYCCD_SUCCESS)
        ui->textBrowser_Info->append("Setup gain " + QString::number(arg1) + " success ...");
    else
        ui->textBrowser_Info->append("Setup gain " + QString::number(arg1) + " failed ...");

    ui->hSlider_Gain->blockSignals(true);
    ui->hSlider_Gain->setValue(arg1);
    ui->hSlider_Gain->blockSignals(false);
}

void QHYCCDDemo::on_hSlider_Offset_valueChanged(int value)
{
    ret = SetQHYCCDParam(camhandle, CONTROL_OFFSET, static_cast<double>(value));
    if(ret == QHYCCD_SUCCESS)
        ui->textBrowser_Info->append("Setup offset " + QString::number(value) + " success ...");
    else
        ui->textBrowser_Info->append("Setup offset " + QString::number(value) + " failed ...");

    ui->spinBox_Offset->blockSignals(true);
    ui->spinBox_Offset->setValue(value);
    ui->spinBox_Offset->blockSignals(false);
}

void QHYCCDDemo::on_spinBox_Offset_valueChanged(int arg1)
{
    ret = SetQHYCCDParam(camhandle, CONTROL_OFFSET, static_cast<double>(arg1));
    if(ret == QHYCCD_SUCCESS)
        ui->textBrowser_Info->append("Setup offset " + QString::number(arg1) + " success ...");
    else
        ui->textBrowser_Info->append("Setup offset " + QString::number(arg1) + " failed ...");

    ui->hSlider_Offset->blockSignals(true);
    ui->hSlider_Offset->setValue(arg1);
    ui->hSlider_Offset->blockSignals(false);
}

// ========================== For Camera 2 ===============================
void QHYCCDDemo::on_pBtn_ScanCamera_2_clicked()
{
    uint32_t num = 0;
    num = ScanQHYCCD();
    if(num == 0)
    {
        ui->textBrowser_Info_2->append("ScanQHYCCD no Camera be found ...");
        return;
    }

    ui->textBrowser_Info_2->append(QString("Found %1 Camera ...").arg(num));
    QApplication::processEvents();

    QStringList idlist;
    char id[40] = {0};
    for(uint32_t i = 0; i < num; i ++)
    {
        memset(id, 0, 40);
        ret = GetQHYCCDId(i, id);
        if(ret == QHYCCD_SUCCESS)
            idlist.append(id);
        else
            ui->textBrowser_Info_2->append(QString("GetQHYCCDId() %1 failed ...").arg(i));
    }

    ui->comboBox_CameraList_2->blockSignals(true);
    ui->comboBox_CameraList_2->clear();
    ui->comboBox_CameraList_2->blockSignals(false);
    ui->comboBox_CameraList_2->addItems(idlist);
}

void QHYCCDDemo::on_comboBox_CameraList_2_currentTextChanged(const QString &arg1)
{
    ret = GetQHYCCDModel(arg1.toLatin1().data(), CamInfo_2.Model);
    if(ret == QHYCCD_ERROR)
    {
        ui->textBrowser_Info_2->append("GetQHYCCDModel failed ...");
        return;
    }

    camhandle_2 = OpenQHYCCD(arg1.toLatin1().data());
    if(camhandle_2 == NULL)
    {
        ui->textBrowser_Info_2->append("OpenQHYCCD failed ...");
        return;
    }

    uint32_t readModeNumber = 0;
    ret = GetQHYCCDNumberOfReadModes(camhandle_2, &readModeNumber);
    QStringList readModeList;
    for(uint32_t i = 0;i < readModeNumber; i++)
    {
        char readModeName[20] = {0};
        ret = GetQHYCCDReadModeName(camhandle_2, i, readModeName);
        if(ret == QHYCCD_SUCCESS)
        {
            readModeList.append(readModeName);
        }
    }

    ui->comboBox_ReadModeList_2->blockSignals(true);
    ui->comboBox_ReadModeList_2->clear();
    ui->comboBox_ReadModeList_2->addItems(readModeList);
    ui->comboBox_ReadModeList_2->blockSignals(false);
}

void QHYCCDDemo::on_pBtn_Connect_2_clicked()
{
    ui->textBrowser_Info_2->append("Start to connect Camera ...");
    QApplication::processEvents();

    if(ui->comboBox_CameraList_2->currentText() == ui->comboBox_CameraList->currentText())
    {
        ui->textBrowser_Info_2->append("Can not open same camera ...");
        return;
    }

    camhandle_2 = OpenQHYCCD(ui->comboBox_CameraList_2->currentText().toLatin1().data());
    if(camhandle_2 == NULL)
    {
        ui->textBrowser_Info_2->append("OpenQHYCCD failed ...");
        return;
    }

    uint32_t readmode = static_cast<uint32_t>(ui->comboBox_ReadModeList_2->currentIndex());
    ret = SetQHYCCDReadMode(camhandle_2, readmode); //Setup Read Mode
    if(ret == QHYCCD_ERROR)
    {
        ui->textBrowser_Info_2->append("SetQHYCCDReadMode failed ...");
        return;
    }

    ret = SetQHYCCDStreamMode(camhandle_2, 0); //Setup Single Mode
    if(ret == QHYCCD_ERROR)
    {
        ui->textBrowser_Info_2->append("SetQHYCCDStreamMode failed ...");
        return;
    }

    ret = InitQHYCCD(camhandle_2);
    if(ret == QHYCCD_ERROR)
    {
        ui->textBrowser_Info_2->append("InitQHYCCD Failed ...");
        return;
    }

    ui->comboBox_Bin_2->blockSignals(true);
    ui->comboBox_Bin_2->clear();
    ui->comboBox_Bin_2->addItem("1x1");
    if(IsQHYCCDControlAvailable(camhandle_2, CAM_BIN2X2MODE) == QHYCCD_SUCCESS) ui->comboBox_Bin_2->addItem("2x2");
    if(IsQHYCCDControlAvailable(camhandle_2, CAM_BIN3X3MODE) == QHYCCD_SUCCESS) ui->comboBox_Bin_2->addItem("3x3");
    if(IsQHYCCDControlAvailable(camhandle_2, CAM_BIN4X4MODE) == QHYCCD_SUCCESS) ui->comboBox_Bin_2->addItem("4x4");
    ui->comboBox_Bin_2->setCurrentText("1x1");
    ui->comboBox_Bin_2->blockSignals(false);

    ret = GetQHYCCDChipInfo(camhandle_2, &CamInfo_2.ChipW, &CamInfo_2.ChipH, &CamInfo_2.ImageW, &CamInfo_2.ImageH, &CamInfo_2.PixelW, &CamInfo_2.PixelH, &CamInfo_2.ImageB);
    if(ret == QHYCCD_ERROR)
    {
        ui->textBrowser_Info_2->append("GetQHYCCDChipInfo failed ...");
        return;
    }

    ret = SetQHYCCDBinMode(camhandle_2, 1, 1);
    if(ret == QHYCCD_ERROR)
    {
        ui->textBrowser_Info_2->append("SetQHYCCDBinMode failed ...");
        return;
    }

    ret = SetQHYCCDResolution(camhandle_2, 0, 0, CamInfo_2.ImageW, CamInfo_2.ImageH);
    if(ret == QHYCCD_ERROR)
    {
        ui->textBrowser_Info_2->append("SetQHYCCDResolution failed ...");
        return;
    }

    ui->comboBox_Format_2->blockSignals(true);
    ui->comboBox_Format_2->clear();
    if(IsQHYCCDControlAvailable(camhandle_2, CAM_16BITS)   == QHYCCD_SUCCESS &&
       IsQHYCCDControlAvailable(camhandle_2, CAM_IS_COLOR) == QHYCCD_SUCCESS) ui->comboBox_Format_2->addItem("RAW16");
    if(IsQHYCCDControlAvailable(camhandle_2, CAM_8BITS)    == QHYCCD_SUCCESS &&
       IsQHYCCDControlAvailable(camhandle_2, CAM_IS_COLOR) == QHYCCD_ERROR)   ui->comboBox_Format_2->addItem("MONO16");
    if(IsQHYCCDControlAvailable(camhandle_2, CAM_IS_COLOR) == QHYCCD_SUCCESS)
        ui->comboBox_Format_2->setCurrentText("RAW16");
    else
        ui->comboBox_Format_2->setCurrentText("MONO16");
    ui->comboBox_Format_2->blockSignals(false);

    ret = SetQHYCCDDebayerOnOff(camhandle_2, false);
    if(ret == QHYCCD_ERROR)
    {
        ui->textBrowser_Info_2->append("SetQHYCCDDebayerOnOff failed ...");
        return;
    }

    ret = SetQHYCCDParam(camhandle_2, CONTROL_TRANSFERBIT, 16);
    if(ret == QHYCCD_ERROR)
    {
        ui->textBrowser_Info_2->append("SetQHYCCDParam(CONTROL_TRANSFERBIT) failed ...");
        return;
    }

    ui->spinBox_Exposure_2->blockSignals(true);
    ui->comboBox_Unit_2->blockSignals(true);
    ui->hSlider_Exposure_2->blockSignals(true);
    ui->spinBox_Exposure_2->setValue(20);
    ui->hSlider_Exposure_2->setValue(20);
    ui->comboBox_Unit_2->clear();
    ui->comboBox_Unit_2->addItem("us");
    ui->comboBox_Unit_2->addItem("ms");
    ui->comboBox_Unit_2->addItem("s");
    ui->comboBox_Unit_2->setCurrentText("ms");
    ui->spinBox_Exposure_2->blockSignals(false);
    ui->comboBox_Unit_2->blockSignals(false);
    ui->hSlider_Exposure_2->blockSignals(false);

    ret = SetQHYCCDParam(camhandle_2, CONTROL_EXPOSURE, 20000.0);
    if(ret == QHYCCD_ERROR)
    {
        ui->textBrowser_Info_2->append("SetQHYCCDParam(CONTROL_EXPOSURE) failed ...");
        return;
    }

    double min, max, step;

    ui->hSlider_Gain_2->blockSignals(true);
    ui->spinBox_Gain_2->blockSignals(true);
    ret = GetQHYCCDParamMinMaxStep(camhandle_2, CONTROL_GAIN, &min, &max, &step);
    ui->hSlider_Gain_2->setMinimum(static_cast<int>(min));
    ui->hSlider_Gain_2->setMaximum(static_cast<int>(max));
    ui->spinBox_Gain_2->setMinimum(static_cast<int>(min));
    ui->spinBox_Gain_2->setMaximum(static_cast<int>(max));
    ui->hSlider_Gain_2->setValue(20);
    ui->spinBox_Gain_2->setValue(20);
    ui->hSlider_Gain_2->blockSignals(false);
    ui->spinBox_Gain_2->blockSignals(false);

    ret = SetQHYCCDParam(camhandle_2, CONTROL_GAIN, 20);
    if(ret == QHYCCD_ERROR)
    {
        ui->textBrowser_Info_2->append("SetQHYCCDParam(CONTROL_GAIN) failed ...");
        return;
    }

    ui->hSlider_Offset_2->blockSignals(true);
    ui->spinBox_Offset_2->blockSignals(true);
    ret = GetQHYCCDParamMinMaxStep(camhandle_2, CONTROL_GAIN, &min, &max, &step);
    ui->hSlider_Offset_2->setMinimum(static_cast<int>(min));
    ui->hSlider_Offset_2->setMaximum(static_cast<int>(max));
    ui->spinBox_Offset_2->setMinimum(static_cast<int>(min));
    ui->spinBox_Offset_2->setMaximum(static_cast<int>(max));
    ui->hSlider_Offset_2->setValue(20);
    ui->spinBox_Offset_2->setValue(20);
    ui->hSlider_Offset_2->blockSignals(false);
    ui->spinBox_Offset_2->blockSignals(false);

    ret = SetQHYCCDParam(camhandle_2, CONTROL_OFFSET, 20);
    if(ret == QHYCCD_ERROR)
    {
        ui->textBrowser_Info_2->append("SetQHYCCDParam(CONTROL_OFFSET) failed ...");
        return;
    }

    CamInfo_2.ImgData = (unsigned char *)malloc(GetQHYCCDMemLength(camhandle_2));
    CamInfo_2.ShowImgData = (unsigned char *)malloc(GetQHYCCDMemLength(camhandle_2));

    this->setWindowTitle(this->windowTitle() + " | " + ui->comboBox_CameraList_2->currentText());
    ui->textBrowser_Info_2->append("Connected Camera,the Camera ID is " + ui->comboBox_CameraList_2->currentText());
}

void QHYCCDDemo::on_pBtn_Disconnect_2_clicked()
{
    if(camhandle_2 == NULL) return;

    ret = CloseQHYCCD(camhandle_2);
    if(ret == QHYCCD_ERROR)
    {
        ui->textBrowser_Info_2->append("CloseQHYCCD() failed ...");
        return;
    }

    camhandle_2 = NULL;
    if(CamInfo_2.ImgData)
    {
        free(CamInfo_2.ImgData);
        CamInfo_2.ImgData = NULL;
    }
    if(CamInfo_2.ShowImgData)
    {
        free(CamInfo_2.ShowImgData);
        CamInfo_2.ShowImgData = NULL;
    }

    this->setWindowTitle(this->windowTitle().remove(" | " + ui->comboBox_CameraList_2->currentText()));
    ui->textBrowser_Info_2->append("Disconnected Camera ...");
}


void QHYCCDDemo::processImageData_2()
{
    ShowImgMutex_2.lock();

    unsigned char *ImageTemp = (unsigned char *)malloc(CamInfo_2.ShowImgW*CamInfo_2.ShowImgH*CamInfo_2.ImgC);

    if(CamInfo_2.ShowImgB == 8 && CamInfo_2.ShowImgC == 1)
    {
        for(uint32_t i = 0; i < CamInfo_2.ShowImgW*CamInfo_2.ShowImgH; i++)
        {
            ImageTemp[i] = CamInfo_2.ShowImgData[i];
        }
    }
    else if(CamInfo_2.ShowImgB == 16 && CamInfo_2.ShowImgC == 1)
    {
        for(uint32_t i = 0; i < CamInfo_2.ShowImgW*CamInfo_2.ShowImgH; i++)
        {
            ImageTemp[i] = CamInfo_2.ShowImgData[2*i+1];
        }
    }
    else if(CamInfo_2.ShowImgB == 8 && CamInfo_2.ShowImgC == 3)
    {
        for(uint32_t i = 0; i < CamInfo_2.ShowImgW*CamInfo_2.ShowImgH*CamInfo_2.ShowImgC; i+=3)
        {
            ImageTemp[i]   = CamInfo_2.ShowImgData[i+2];
            ImageTemp[i+1] = CamInfo_2.ShowImgData[i+1];
            ImageTemp[i+2] = CamInfo_2.ShowImgData[i];
        }
    }
    else if(CamInfo_2.ShowImgB == 16 && CamInfo_2.ShowImgC == 3)
    {
        for(uint32_t i = 0; i < CamInfo_2.ShowImgW*CamInfo_2.ShowImgH*CamInfo_2.ShowImgC; i+=3)
        {
            ImageTemp[i]   = CamInfo_2.ShowImgData[2*i+5];
            ImageTemp[i+1] = CamInfo_2.ShowImgData[2*i+3];
            ImageTemp[i+2] = CamInfo_2.ShowImgData[2*i+1];
        }
    }

    QImage::Format format = CamInfo_2.ShowImgC == 1 ? QImage::Format_Grayscale8 : QImage::Format_RGB888;
    QImage *ImageOri = new QImage(ImageTemp, CamInfo_2.ShowImgW, CamInfo_2.ShowImgH, CamInfo_2.ShowImgW*CamInfo_2.ShowImgC, format);
    QImage ImageShow = ImageOri->scaled(ui->label_Image_2->width(), ui->label_Image_2->height(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
    ui->label_Image_2->setPixmap(QPixmap::fromImage(ImageShow));

    delete ImageOri;
    free(ImageTemp);

    ShowImgMutex_2.unlock();

    ui->label_Frame_2->setText("Frame : " + QString::number(CamInfo_2.Frame));
    ui->textBrowser_Info_2->append("Capture image success ...");

    if(ui->checkBox_AutoSave_2->isChecked())
    {
        QString savePath = QCoreApplication::applicationDirPath();
        QString fileName = savePath + "/" + CamInfo_2.Model + "-" +
                QDateTime::currentDateTime().toString("yyyyMMddhhmmss") + ".fit";
        FITwrite(fileName, CamInfo_2.ShowImgW, CamInfo_2.ShowImgH, CamInfo_2.ShowImgB, CamInfo_2.ShowImgC, CamInfo_2.ShowImgData);
        ui->textBrowser_Info_2->append("Save as to FIT ...");
    }
}

void QHYCCDDemo::on_pBtn_Capture_2_clicked()
{
    if(camhandle_2 == NULL) return;

    //for some cameras, must setup bin and resolution in every capture
    this->on_comboBox_Bin_2_currentTextChanged(ui->comboBox_Bin_2->currentText());

    ui->textBrowser_Info_2->append("Start capturing ...");
    QApplication::processEvents();

    ret = ExpQHYCCDSingleFrame(camhandle_2);
    if(ret == QHYCCD_ERROR)
    {
        ui->textBrowser_Info_2->append("ExpQHYCCDSingleFrame() failed ...");
        return;
    }

    singleThread_2 = new SingleModeThread_2(this);
    connect(singleThread_2, SIGNAL(finished()), singleThread_2, SLOT(deleteLater()));
    connect(singleThread_2, SIGNAL(gotImageData_2()), this, SLOT(processImageData_2()));
    singleThread_2->start();
}


void QHYCCDDemo::on_pBtn_Cancel_2_clicked()
{
    if(camhandle_2 == NULL) return;

    ret = CancelQHYCCDExposingAndReadout(camhandle_2);
    if(ret == QHYCCD_ERROR)
    {
        ui->textBrowser_Info_2->append("CancelQHYCCDExposingAndReadout() failed ...");
        return;
    }
}


void QHYCCDDemo::on_comboBox_Format_2_currentTextChanged(const QString &arg1)
{
    if(arg1 == "RAW8" || arg1 == "MONO8")
    {
        ret = SetQHYCCDDebayerOnOff(camhandle_2, false);
        ret = SetQHYCCDParam(camhandle_2, CONTROL_TRANSFERBIT, 8.0);
    }
    else if(arg1 == "RAW16" || arg1 == "MONO16")
    {
        ret = SetQHYCCDDebayerOnOff(camhandle_2, false);
        ret = SetQHYCCDParam(camhandle_2, CONTROL_TRANSFERBIT, 16.0);
    }
    else if(arg1 == "RGB8")
    {
        ret = SetQHYCCDDebayerOnOff(camhandle_2, true);
        ret = SetQHYCCDParam(camhandle_2, CONTROL_TRANSFERBIT, 8.0);
    }

    if(ret == QHYCCD_SUCCESS)
        ui->textBrowser_Info_2->append("Setup " + arg1 + " success ...");
    else
        ui->textBrowser_Info_2->append("Setup " + arg1 + " failed ...");
}

void QHYCCDDemo::on_comboBox_Bin_2_currentTextChanged(const QString &arg1)
{
    if(arg1 == "1x1")
    {
        ret = SetQHYCCDBinMode(camhandle_2, 1, 1);
        ret = SetQHYCCDResolution(camhandle_2, 0, 0, CamInfo_2.ImageW, CamInfo_2.ImageH);
    }
    if(arg1 == "2x2")
    {
        ret = SetQHYCCDBinMode(camhandle_2, 2, 2);
        ret = SetQHYCCDResolution(camhandle_2, 0, 0, CamInfo_2.ImageW / 2, CamInfo_2.ImageH / 2);
    }
    if(arg1 == "3x3")
    {
        ret = SetQHYCCDBinMode(camhandle_2, 3, 3);
        ret = SetQHYCCDResolution(camhandle_2, 0, 0, CamInfo_2.ImageW / 3, CamInfo_2.ImageH / 3);
    }
    if(arg1 == "4x4")
    {
        ret = SetQHYCCDBinMode(camhandle_2, 4, 4);
        ret = SetQHYCCDResolution(camhandle_2, 0, 0, CamInfo_2.ImageW / 4, CamInfo_2.ImageH / 4);
    }

    if(ret == QHYCCD_SUCCESS)
        ui->textBrowser_Info_2->append("Setup " + arg1 + " success ...");
    else
        ui->textBrowser_Info_2->append("Setup " + arg1 + " failed ...");
}

void QHYCCDDemo::on_spinBox_Exposure_2_valueChanged(int arg1)
{
    double unit = 1.0;
    if(ui->comboBox_Unit_2->currentText() == "us")      unit = 1.0;
    else if(ui->comboBox_Unit_2->currentText() == "ms") unit = 1000.0;
    else if(ui->comboBox_Unit_2->currentText() == "s")  unit = 1000000.0;

    ret = SetQHYCCDParam(camhandle_2, CONTROL_EXPOSURE, static_cast<double>(arg1)*unit);
    if(ret == QHYCCD_SUCCESS)
        ui->textBrowser_Info_2->append("Setup exposure " + QString::number(arg1) + ui->comboBox_Unit_2->currentText() + " success ...");
    else
        ui->textBrowser_Info_2->append("Setup exposure " + QString::number(arg1) + ui->comboBox_Unit_2->currentText() + " failed ...");

    ui->hSlider_Exposure_2->blockSignals(true);
    ui->hSlider_Exposure_2->setValue(arg1);
    ui->hSlider_Exposure_2->blockSignals(false);

}

void QHYCCDDemo::on_comboBox_Unit_2_currentTextChanged(const QString &arg1)
{
    double unit = 1.0;
    if(arg1 == "us")      unit = 1.0;
    else if(arg1 == "ms") unit = 1000.0;
    else if(arg1 == "s")  unit = 1000000.0;
    int exp = ui->spinBox_Exposure_2->value();

    ret = SetQHYCCDParam(camhandle_2, CONTROL_EXPOSURE, static_cast<double>(exp)*unit);
    if(ret == QHYCCD_SUCCESS)
        ui->textBrowser_Info_2->append("Setup exposure " + QString::number(exp) + arg1 + " success ...");
    else
        ui->textBrowser_Info_2->append("Setup exposure " + QString::number(exp) + arg1 + " failed ...");
}

void QHYCCDDemo::on_hSlider_Exposure_2_valueChanged(int value)
{
    double unit = 1.0;
    if(ui->comboBox_Unit_2->currentText() == "us")      unit = 1.0;
    else if(ui->comboBox_Unit_2->currentText() == "ms") unit = 1000.0;
    else if(ui->comboBox_Unit_2->currentText() == "s")  unit = 1000000.0;

    ret = SetQHYCCDParam(camhandle_2, CONTROL_EXPOSURE, static_cast<double>(value)*unit);
    if(ret == QHYCCD_SUCCESS)
        ui->textBrowser_Info_2->append("Setup exposure " + QString::number(value) + ui->comboBox_Unit_2->currentText() + " success ...");
    else
        ui->textBrowser_Info_2->append("Setup exposure " + QString::number(value) + ui->comboBox_Unit_2->currentText() + " failed ...");

    ui->spinBox_Exposure_2->blockSignals(true);
    ui->spinBox_Exposure_2->setValue(value);
    ui->spinBox_Exposure_2->blockSignals(false);
}

void QHYCCDDemo::on_hSlider_Gain_2_valueChanged(int value)
{
    ret = SetQHYCCDParam(camhandle_2, CONTROL_GAIN, static_cast<double>(value));
    if(ret == QHYCCD_SUCCESS)
        ui->textBrowser_Info_2->append("Setup gain " + QString::number(value) + " success ...");
    else
        ui->textBrowser_Info_2->append("Setup gain " + QString::number(value) + " failed ...");

    ui->spinBox_Gain_2->blockSignals(true);
    ui->spinBox_Gain_2->setValue(value);
    ui->spinBox_Gain_2->blockSignals(false);
}

void QHYCCDDemo::on_spinBox_Gain_2_valueChanged(int arg1)
{
    ret = SetQHYCCDParam(camhandle_2, CONTROL_GAIN, static_cast<double>(arg1));
    if(ret == QHYCCD_SUCCESS)
        ui->textBrowser_Info_2->append("Setup gain " + QString::number(arg1) + " success ...");
    else
        ui->textBrowser_Info_2->append("Setup gain " + QString::number(arg1) + " failed ...");

    ui->hSlider_Gain_2->blockSignals(true);
    ui->hSlider_Gain_2->setValue(arg1);
    ui->hSlider_Gain_2->blockSignals(false);
}

void QHYCCDDemo::on_hSlider_Offset_2_valueChanged(int value)
{
    ret = SetQHYCCDParam(camhandle_2, CONTROL_OFFSET, static_cast<double>(value));
    if(ret == QHYCCD_SUCCESS)
        ui->textBrowser_Info_2->append("Setup offset " + QString::number(value) + " success ...");
    else
        ui->textBrowser_Info_2->append("Setup offset " + QString::number(value) + " failed ...");

    ui->spinBox_Offset_2->blockSignals(true);
    ui->spinBox_Offset_2->setValue(value);
    ui->spinBox_Offset_2->blockSignals(false);
}

void QHYCCDDemo::on_spinBox_Offset_2_valueChanged(int arg1)
{
    ret = SetQHYCCDParam(camhandle_2, CONTROL_OFFSET, static_cast<double>(arg1));
    if(ret == QHYCCD_SUCCESS)
        ui->textBrowser_Info_2->append("Setup offset " + QString::number(arg1) + " success ...");
    else
        ui->textBrowser_Info_2->append("Setup offset " + QString::number(arg1) + " failed ...");

    ui->hSlider_Offset_2->blockSignals(true);
    ui->hSlider_Offset_2->setValue(arg1);
    ui->hSlider_Offset_2->blockSignals(false);
}
